#!@@PERL@@
#
# machine.cgi						Author: Rich West
#							rwest@wesmo.com
#
# Acting as a GUI front end to all of the core system files for networked
# machines, this CGI script provides an interface for admins to modify
# (add/delete) information from the MySQL machine databases.
#
# It also provides a secure and simplified way of management over pushing
# out that information to the respective servers on the network to make those
# changes reflect across all systems.
#
# $Header: /home/rwest/work/MID/src/RCS/machine.cgi,v 1.8 2001/02/14 18:31:45 rwest Exp $
################################################################################

################################################################################
#
# The name of this script.
#
################################################################################
$VERSION = "@@VERSION@@";
$my_name=`basename $0`;
chop($my_name);

################################################################################
#
# The modules we require.
#
################################################################################
use Mysql;
use CGI::Cookie;

&main();

sub db_settings
{
################################################################################
#
# Our Database Information
#
################################################################################
$host = "@@DBHOST@@";
$dbname = "@@DB@@";
$user = "@@DBUSER@@";
$pass = "@@DBPASS@@";
}
 
################################################################################
#
# Grab all of our defaults from the database 'conf' table.
#
################################################################################
sub defaults
{
   local($qs, @row);
   $qs = "SELECT domainname,named,yp,tmp,start_range,end_range,dhcpd FROM conf";
   $sth = $dbh->Query($qs)
      or warn "[defaults] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   $DOMAINNAME = $row[0];
   @RANGE=($row[4],$row[5]);
   $NAMED_DIR=$row[1];
   $NIS_DIR=$row[2];
   $TMP=$row[3];
   $DHCPD_DIR=$row[6];

   $noshow{"OS"} = 1;
   $noshow{"comment"} = 1;
}

sub format_styles
{
################################################################################
#
# Our Format Styles
#
################################################################################
format DNS_A =
@<<<<<<<<<<<<<<<<<<<<	IN	A	@<<<<<<<<<<<<<<<
$name,$ip
.

format DNS_C =
@<<<<<<<<<<<<<<<<<<<<	IN	CNAME	@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
$alias,$name
.

format DNS_REV =
@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<   IN   PTR   @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
$revip,$name
.
}

################################################################################
#
# Connect to the Database
#
################################################################################
sub connectdb
{
   local($dbname) = @_;

   if (!($dbh = Mysql->Connect($host, $dbname, $user, $pass,)))
   {
      die "[connectdb] Database connect failed: $Mysql::db_errstr\n";
   }
}

################################################################################
#
# Determine what to do based upon the environmental variables.
#
################################################################################
sub main
{
   &web_get();
   if ($FORM{'changelog'})
   {
      &set_up_page("CHANGE LOG for Version $VERSION");
      &show_changelog();
      &end_page();
      exit;
   }
   if ($FORM{'logout'})
   {
      &logout();
      exit;
   }
   &db_settings();
   &connectdb("@@DB@@");
   &get_user();
   if ($FORM{'help'})
   {
      &set_up_page("Proper Syntax and Limitations for $FORM{'describe'}");
      &help($FORM{'describe'});
      &end_page();
      exit;
   }
   &defaults();
   &format_styles();
   @fields=&get_db_fields();
   if ($FORM{"function"} eq "Query")
   {
      $entry = &search_db($FORM{"Query"}, @fields);
      if (!$entry) { &show_options(1, @fields); }
      &set_up_page("Query Results for '$FORM{'Query'}':");
      &display($entry,"query", @fields);
      &end_page();
      exit;
   }
   if ($FORM{"function"} eq "Modify")
   {
      $entry = &search_db($FORM{"Query"}, @fields);
      if (!$entry) { &show_options(1, @fields); }
      &modify($entry, @fields);
   }
   if ($FORM{"function"} eq "Add")
   {
      &new_entry($FORM{"function"}, @fields);
   }
   if ($FORM{"function"} eq "Delete")
   {
      $entry = &search_db($FORM{"Query"}, @fields);
      if (!$entry) { &show_options(1, @fields); }
      &remove_entry($entry,@fields);
   }
   if ($FORM{"Print"})
   {
      &dump_database(@fields);
   }
   if ($FORM{"submit"} eq "modify")
   {
      if (!&errors(@fields))
      {
         $FORM{'machinename'} = &clean_entry($FORM{'machinename'},0);
         $FORM{'alias'} = &clean_entry($FORM{'alias'},0);
         $FORM{'IP'} = &clean_entry($FORM{'IP'},1);
         &delete_entry();
         &add_entry();
         $error=2;
      }
      else
      {
         &display_error();
      }
   }
   if ($FORM{"submit"} eq "delete")
   {
      if (!&errors(@fields))
      {
         &delete_entry();
         $error=2;
      }
      else
      {
         &display_error();
      }
   }
   if ($FORM{"submit"} eq "add")
   {
      $FORM{'machinename'} = &clean_entry($FORM{'machinename'},0);
      $FORM{'alias'} = &clean_entry($FORM{'alias'},0);
      $FORM{'IP'} = &clean_entry($FORM{'IP'},1);
      $error=20 if (&exists($FORM{'machinename'},"machinename"));
      $error=21 if ( &exists($FORM{'IP'}, "IP") && !$error);
      if ($FORM{'hardware_addr'})
      {
         $error=22 if (&exists($FORM{'hardware_addr'}, "hardware_addr") && !$error);
      }
      $error=23 if ( (!$FORM{'machinename'}) || (!$FORM{'IP'}));
      if (!$error)
      {
         &add_entry();
         $error=2;
      }
   }
   if ($FORM{'PushMaps'})
   {
      &update_maps();
      &push_maps();
      $error=3;
   }
   if ($FORM{'Reports'})
   {
      &subnet_usage();
   }
   if ($FORM{'Admin'})
   {
      &administer();
      exit;
   }
   if ($FORM{'UPDATE_CONF'})
   {
      if ( $FORM{'Update'} eq "Cancel")
      {
         $error=5;
      }
      else
      {
         &update_config;
         $error=4;
      }
   }
   &show_options($error,@fields);
}
 
################################################################################
#
# Print the top of the page, starting things off.
#
################################################################################
sub set_up_page()
{
   ($title) = @_;

   print <<END
Content-type: text/html


<HTML>
<HEAD>
   <TITLE>DNS Machine Database</TITLE>
   <SCRIPT LANGUAGE="Javascript">

   <!-- //
   function openDescription(placement, height)
   {
      if (!height)
         height = 240;
            
       window.name='cd0ContentFrame';
END
;
   print "var desString = '$my_name#' + placement;";
   print <<END
       var desWin = window.open(desString,'description', 'scrollbars=yes,location=no,status=no,menubar=no,height='+height+',width=420,resizable=yes');
       desWin.focus();
    } // openDescription()
   // -->
</SCRIPT>
</HEAD>
<BODY TOPMARGIN="0" LEFTMARGIN="0" MARGINWIDTH="0" MARGINHEIGHT="0" BGCOLOR=#ffffff>
END
;
   if (!$title)
   {
      print "<CENTER><IMG SRC=\"@@IMAGES@@/inventory.jpg\"></CENTER>";
   }
   else
   {
      print "<H1>$title</H1>";
   }
   print "<HR SIZE=1 NOSHADE>\n";
}

################################################################################
#
# Print the bottom of the page, closing things out.
#
################################################################################
sub end_page()
{
   local($show) = @_;

   print <<END
<HR SIZE=1 NOSHADE>
<TABLE WIDTH=100% CELLSPACING=0 CELLPADDING=0 BORDER=0>
<TR>
   <TD ALIGN=LEFT VALIGN=TOP WIDTH=45%>
<FONT SIZE=1>
&nbsp;&nbsp; This page, and all contents, &copy; 1999-2001 by <A HREF="mailto:rwest\@wesmo.com">Rich West</A>
</FONT>
   </TD>
END
;
   if ($show)
   {
      print <<END
   <TD VALIGN=TOP ALIGN=CENTER>
      <FONT SIZE=1><A HREF="$my_name?logout=1">Log Out</A>
   </FONT>
   </TD>
END
;
   }
   print <<END
   <TD ALIGN=RIGHT VALIGN=TOP WIDTH=45%>
<FONT SIZE=1>
<P ALIGN=RIGHT>Version: <A HREF="$my_name?changelog=1">$VERSION</A>
</FONT>
   </TD>
</TR>
</TABLE>
</BODY>
</HTML>
END
;
}

################################################################################
#
# Display a login screen, asking for a name and password.  It is rudimentary,
# but it works.
#
################################################################################
sub get_user()
{
   %cookies = fetch CGI::Cookie;
   if ( $cookies{'MachineDatabase'} )
   {
#      $set_cookie = new CGI::Cookie(-name => 'MachineDatabase',
#          -value => $USER,
#          -expires => '+8h'
#          -secure => 1);
#      print "Set-Cookie: $set_cookie\n";
      return;
   }
   if ($FORM{'user'})
   {
      $query_string = "SELECT password FROM users WHERE name='$FORM{'user'}'";
      $sth = $dbh->Query($query_string)
         or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string\n";
      @row=$sth->FetchRow;
      $stored_password = $row[0];
      if ($stored_password)
      {
         $seed = (split(//, $stored_password))[0] . (split(//, $stored_password))[1];
         $new = crypt($FORM{'password'}, $seed);
         if ($stored_password eq $new)
         {
            $USER = $FORM{'user'};
            $PASSWORD = $FORM{'password'};
            $set_cookie = new CGI::Cookie(-name => 'MachineDatabase',
                -value => $USER,
                -expires => '+8h');
            print "Set-Cookie: $set_cookie\n";
            return;
         }
      }
   }
#   if (($FORM{'user'} ne $USER) || ($FORM{'password'} ne $PASSWORD))
#   {
      &set_up_page("<CENTER>You MUST Login in order to Access this Database.</CENTER>");
      print <<END
<FORM METHOD="POST"
        ACTION="/cgi-bin/$my_name">
<CENTER>
<TABLE BGCOLOR="#3a1919" CELLPADDING=0 CELLSPACING=0 BORDER=0>
<TR>
<TD COLSPAN=3><IMG SRC="@@IMAGES@@/top.gif" BORDER=0></TD>
</TR>
<TR>
<TD><IMG SRC="@@IMAGES@@/login.gif" BORDER=0></TD>
<TD><INPUT TYPE="text" NAME="user" SIZE=10></TD>
<TD>&nbsp;</TD>
</TR>
<TR>
<TD>
<IMG SRC="@@IMAGES@@/password.gif" BORDER=0>
</TD>
<TD><INPUT TYPE="password" NAME="password" SIZE=10></TD>
<TD>&nbsp;</TD>
</TR>
<TR>
<TD ALIGN=CENTER COLSPAN=3><INPUT TYPE="image" BORDER=0 SRC="@@IMAGES@@/log_in.gif" ALT="Log In!"></TD>
</TR>
<TR>
<TD COLSPAN=3><IMG SRC="@@IMAGES@@/bottom.gif" BORDER=0></TD>
</TR>
</TABLE>
</FORM>
</CENTER>
END
;
      &end_page(0);
      exit;
#   }
}

################################################################################
#
# Blow away the user's cookie then bring them back to the login screen.
#
################################################################################
sub logout
{
   %cookies = fetch CGI::Cookie;
   if (exists $cookies{'MachineDatabase'})
   {
      my $pass_cookie;
      $pass_cookie = $cookies{'MachineDatabase'};
      $pass_cookie->expires('now');
      $pass_cookie->value('');
      print "Set-Cookie: ", $pass_cookie, "\n";
   }
   print "Content-type: text/html\n\n";
   print " <HTML> <HEAD> <META HTTP-EQUIV=\"REFRESH\" CONTENT=\"2 ; URL=$my_name\">\n</HEAD>";
   print "<CENTER><H2><FONT COLOR=RED>Processing Log Out...</FONT></H2></CENTER>\n";
   print "</BODY>";
}
################################################################################
#
# A rather standard routine to grab the environmental variables and drop them
# into a variable form of $FORM{'name'}.
#
################################################################################
sub web_get()
{
   ######cut here ###################################################
   # Get the input
   if ($ENV{'CONTENT_LENGTH'})
   {
      read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
   }
   # Split the name-value pairs
   if ($buffer) {
       @pairs = split(/&/, $buffer);
   }
   else {
 
       @pairs = split(/&/,$ENV{'QUERY_STRING'});
   }
 
   foreach $pair (@pairs)
   {
       ($name, $value) = split(/=/, $pair);
       # Un-Webify plus signs and %-encoding
       $value =~ tr/+/ /;
       $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
       $value =~ s/&lt/</g;
       $value =~ s/&gt/>/g;
       # Stop people from using subshells to execute commands
       # Not a big deal when using sendmail, but very important
       # when using UCB mail (aka mailx).
       $value =~ s/~!/ ~!/g;
       # Uncomment for debugging purposes
       #print "A gnarled troll tells you that  was set to $value<br>";
       $FORM{$name} = $value;
   }
#### ----cut here
}

################################################################################
#
# In a generic and dynamic format, show a pull-down selection of all of the
# available fields.
#
################################################################################
sub show_fields
{
   local($type,@fields) = @_;

   print "<SELECT NAME=\"$type\">\n";
   foreach $count (0..$#fields)
   {
     next if ($noshow{$fields[$count]});
     print "<OPTION";
     if ($FORM{$type} eq $fields[$count]) { print " SELECTED"; }
     print " VALUE=\"$fields[$count]\"> $fields[$count]\n";
   }
   print "</SELECT>\n";
}

################################################################################
#
# Generate the initial welcome page, which shows all of the possible options
# and searches that can be done on the master DNS database.
#
################################################################################
sub show_options
{
   local($error, @fields)=@_;

#   &set_up_page("DNS Machine Database Options"); # now a graphic
   &set_up_page("");
   &notify($error);
   print <<END
<CENTER>
<TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0 WIDTH=80%>
<TR>
   <TD COLSPAN=3>
<FORM METHOD="POST" ACTION="/cgi-bin/$my_name">
<SELECT NAME="function">
<OPTION VALUE="Query">Query
<OPTION VALUE="Modify">Modify
<OPTION VALUE="Delete">Delete
</SELECT>
 the Database for the Machine with the
END
;
   &show_fields("query_type",@fields);
print <<END
:
<INPUT TYPE="text" NAME="Query" value="$FORM{'Query'}" SIZE=8>
<INPUT TYPE="image" BORDER=0 SRC="@@IMAGES@@/go.gif" ALT="Go!">
</FORM>
   </TD>
</TR>
<TR>
   <TD COLSPAN=3>
<FORM METHOD="POST"
        ACTION="/cgi-bin/$my_name">
<INPUT TYPE="submit" name="function" VALUE="Add">
a new entry to the database in the:
END
;
   %subs=&get_subnets();
   foreach $SUB (sort numerically keys %subs)
   {
      $subnets[++$#subnets] = "$SUB";
   }
   &show_fields("subnet",@subnets);
print <<END
subnet.
<FONT SIZE=-2><INPUT TYPE="checkbox" name="specific">with a Specific IP</FONT>
</FORM>
   </TD>
</TR>
<TR>
   <TD COLSPAN=3>
<FORM METHOD="POST"
        ACTION="/cgi-bin/$my_name">
<INPUT TYPE="submit" name="Print" VALUE="Display">
 the 
END
;
   $subnets[++$#subnets] = "Entire";
   &show_fields("Range", @subnets);
   print " Database, sorting by ";
   &show_fields("query_type",@fields);
print <<END
</FORM>
   </TD>
</TR>
<TR>
   <TD ALIGN=LEFT>
<FORM METHOD="POST"
        ACTION="/cgi-bin/$my_name">
Show the Subnet Usage
<INPUT TYPE="submit" name="Reports" VALUE="Reports">
</FORM>
   </TD>
   <TD ALIGN=CENTER>
<FORM METHOD="POST"
        ACTION="/cgi-bin/$my_name">
<INPUT TYPE="submit" name="PushMaps" VALUE="Push">
Maps
</FORM>
   </TD>
   <TD ALIGN=RIGHT>
<FORM METHOD="POST"
        ACTION="/cgi-bin/$my_name">
<INPUT TYPE="submit" name="Admin" VALUE="Administer"> Configuration/Settings
Maps
</FORM>
   </TD>
</TR>
</TABLE>
</CENTER>
END
;
   &end_page(1);
   exit;
}

################################################################################
#
# Search through the entire database and find the item specified.  It must
# match exactly.
#
################################################################################
sub search_db
{
   local($entry, @fields) = @_;

   if ($FORM{'query_type'} eq "alias")
   {
      $id = &get_alias_id($entry); 
   }
   else
   {
      $id = &get_id($entry);
   }
   if ($id)
   {
      $machine_info = &get_ip($id);
      $alias = &get_aliases($id);
      $owner = &get_owner($id);
      return ("$machine_info\;$alias\;$owner");
   }
   return(0);
}

################################################################################
#
# Get the id from the aliases database.
#
################################################################################
sub get_alias_id
{
   local($entry) = @_;
   local($id);

   $query_string = "SELECT id FROM aliases WHERE $FORM{'query_type'}='$entry'";
   $sth = $dbh->Query($query_string)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   return ($row[0]);
}

################################################################################
#
# Get the database id for this entry.
#
################################################################################
sub get_id
{
   local($entry) = @_;
   local($query_string);

   if ( ($FORM{'submit'} eq "add") || ($FORM{'submit'} eq "modify") )
   {
      $query_string="SELECT id FROM name WHERE $FORM{query_type}='$entry'";
   }
   else
   {
      $query_string="select name.id FROM name,owner WHERE $FORM{query_type}='$entry' AND name.id=owner.id"; 
   }
   $sth = $dbh->Query($query_string)
      or warn "[get_id] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   return ($row[0]);
}

################################################################################
#
# Get the IP and other machine specific information
#
################################################################################
sub get_ip
{
   local($id) = @_;
   local($query_string);

   $query_string="SELECT IP,machinename,hardware_addr,OS FROM name WHERE id=$id";
   $sth = $dbh->Query($query_string)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   return ("$row[0]\;$row[1]\;$row[2]\;$row[3]");
}

################################################################################
#
# Get all of the owner information.
#
################################################################################
sub get_owner
{
   local($id) = @_;
   local($query_string);

   $query_string="SELECT ownername,location,comment FROM owner WHERE id=$id";
   $sth = $dbh->Query($query_string)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   return ("$row[0]\;$row[1]\;$row[2]");
}

################################################################################
#
# Get all of the aliases for this entry.
#
################################################################################
sub get_aliases
{
   local($id) = @_;
   local($query_string);
                                    
   $query_string="SELECT alias FROM aliases WHERE id=$id";
   $sth = $dbh->Query($query_string)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   $aliases = $row[0];
   while (@row=$sth->FetchRow)
   {
      $aliases .= ",$row[0]";
   }
   return ("$aliases");
}

################################################################################
#
# A semi-generic routine which sets up the display for modifying, deleting,
# or adding an entry.
#
################################################################################
sub display
{
   local($entry,$type, @fields) = @_;
   local($count, @split_entry,@row,$sth,$qs);

   foreach (split("\;", $entry))
   {
      $split_entry[$count]=$_;
      $count++;
   }
   $count=0;
   print "<TABLE>";
   foreach $count (0..$#fields)
   {
      if (($fields[$count] eq "IP") && ($FORM{'function'} eq "Add") && (!$FORM{'specific'}))
      {
         next;
      }
      print "<TR>\n   <TD>\n   $fields[$count] :\n   </TD>\n   <TD>\n";
      if (($type eq "modify") || ($type eq "new"))
      {
         if ($fields[$count] eq "OS")
         {
            $qs = "SELECT * FROM OS";
            $sth = $dbh->Query($qs)
               or warn "[display] Query had some problem: $Mysql::db_errstr\n$qs\n";
            print "<SELECT NAME=\"OS\">\n";
            print "<OPTION VALUE=\"0\">None\n";
            while(@row=$sth->FetchRow)
            {
               print "<OPTION VALUE=\"$row[0]\"";
               print "SELECTED" if ($split_entry[$count] eq $row[0]);
               print ">$row[1]\n";
            }
            print "</SELECT>\n";
         }
         else
         {
            print qq#   <INPUT TYPE="text" NAME="$fields[$count]" VALUE="$split_entry[$count]" SIZE=16>#;
         }
         print qq#<A HREF="javascript://" onClick="openDescription('?help=1&describe=$fields[$count]');"><img src="@@IMAGES@@/descriptionButton.gif" width="10" height="10" border="0" alt="Click here for more information"></a>#;
      }
      else
      {
         if ($fields[$count] eq "OS")
         {
            $qs = "SELECT name FROM OS WHERE id='$split_entry[$count]'";
            $sth = $dbh->Query($qs)
               or warn "[display] Query had some problem: $Mysql::db_errstr\n$qs\n";
            @row=$sth->FetchRow;
            print "$row[0] \n";
         }
         else
         {
            print "$split_entry[$count] \n";
         }
         print "<INPUT TYPE=\"hidden\" NAME=\"$fields[$count]\" VALUE=\"$split_entry[$count]\">\n";
      }
      print "</TD>\n   </TR>\n";
      $count++;
   }
   print "</TABLE>\n";
}
 
################################################################################
#
# Get all of the valid fields in the database.  This is grabbed from the first
# line in the database, and is entirely dynamic.
#
################################################################################
sub get_db_fields
{
   local($field,$count);
   local($output,$i,$j,@row2,$table,@tables,@db);

   $i=0;
#   $query_string="SHOW TABLES";
#   $sth = $dbh->Query($query_string)
#      or warn "[get_db_fields] Query had some problem: $Mysql::db_errstr\n$query_string\n";
#   while(@row=$sth->FetchRow)
   @tables = ("name", "aliases", "owner");
   for ($j=0; $j<=$#tables; $j++)
   {
      $table = $row[0];
      $table = $tables[$j];
      $query_string2="SHOW FIELDS FROM $table";
      $sth2 = $dbh->Query($query_string2)
      or warn "[get_db_fields] Query had some problem: $Mysql::db_errstr\n$query_string\n";
      while(@row2=$sth2->FetchRow)
      {
         next if ($row2[0] eq "id");
         $field[$i] = $row2[0];
         $db[$i] = $table;
         $i++;
      }
   }
   return (@field);
}

################################################################################
#
# Generate the page to get the information to modify an entry.
#
################################################################################
sub modify
{
   local($entry, @fields) = @_;

   &set_up_page("Modify '$FORM{'Query'}':");
   print <<END
<FORM METHOD="POST"
        ACTION="/cgi-bin/$my_name">
<INPUT TYPE="hidden" name="submit" value="modify">
<INPUT TYPE="hidden" name="delete_value" value=\"$FORM{'Query'}\">
<INPUT TYPE="hidden" name="query_type" value=\"$FORM{'query_type'}\">
END
;
   &display($entry,"modify", @fields);
   print <<END
<INPUT TYPE="submit" VALUE="Commit Modification">
</FORM>
END
;
   &end_page(1);
   exit;
}

################################################################################
#
# Generate the page to get the information to add an entry.
#
################################################################################
sub new_entry
{
   local($entry, @fields)=@_;
   local($ip);

   &set_up_page("New Entry Form");
   print <<END
<FORM METHOD="POST"
        ACTION="/cgi-bin/$my_name">
<INPUT TYPE="hidden" name="submit" value="add">
END
;
   print qq#<INPUT TYPE="hidden" name="specific" value="true">\n# if ($FORM{'specific'});
   &display("","modify", @fields);
   $ip=&new_ip($FORM{subnet}) if (!$FORM{'specific'});
   print qq#<INPUT TYPE="hidden" name="IP" value="$ip">\n# if (!$FORM{'specific'});
   print qq#<INPUT TYPE="submit" VALUE="Commit Addition">\n</FORM>#;
   &end_page(1);
   exit;
}

################################################################################
#
# Strip out some of the more popular invalid characters.
#
################################################################################
sub clean_entry
{
   local ($variable,$flag) = @_;

   $_ = $variable;
   if ($flag)
   {
      s/\ //g;
   }
   else
   {
      s/\ /-/g;
      s/\_/-/g;
      s/\*/-/g;
      s/\&/-/g;
      s/\%/-/g;
      s/\$/-/g;
      s/\#/-/g;
      s/\@/-/g;
      s/\!/-/g;
      s/\`/-/g;
      s/\~/-/g;
      s/\'/-/g;
      s/\"/-/g;
      s/\:/-/g;
      s/\;/-/g;
   }
   return($_);
}
################################################################################
#
# Physically add an entry into the database, defined by what was previously
# entered into the on-line form (the subnet).  As of this time, no error
# checking is done on the data entered.
#
################################################################################
sub add_entry
{
   local($sth, $query_string);

   $query_string = "INSERT INTO name VALUES ('', '$FORM{'IP'}', '$FORM{'machinename'}', '$FORM{'hardware_addr'}', '$FORM{'OS'}')";
   $sth = $dbh->Query($query_string)
      or warn "[add_entry] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   $FORM{query_type}="IP";
   $id = &get_id($FORM{'IP'});
   if ($FORM{'alias'})
   {
      @aliases = split(/,/, $FORM{'alias'});
      for($i=0; $i<=($#aliases); $i++)
      {
         $query_string = "INSERT INTO aliases VALUES ('$id', '$aliases[$i]')";
         $sth = $dbh->Query($query_string)
            or warn "[add_entry] Query had some problem: $Mysql::db_errstr\n$query_string\n";
      }
   }
   $query_string = "INSERT INTO owner VALUES ('$id', '$FORM{'ownername'}', '$FORM{'location'}', '$FORM{'comment'}')";
   $sth = $dbh->Query($query_string)
      or warn "[add_entry] Query had some problem: $Mysql::db_errstr\n$query_string\n";
}

################################################################################
#
# Physically remove an entry from the database, defined by what was previously
# entered into the on-line form.  The item must match exactly.
#
################################################################################
sub delete_entry
{
   local($split_entry,$count);

   $id=&get_id($FORM{'delete_value'});
   $query_string="DELETE FROM name WHERE id='$id'";
   $sth = $dbh->Query($query_string)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string
\n";
   $query_string="DELETE FROM aliases WHERE id='$id'";
   $sth = $dbh->Query($query_string)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string
\n";
   $query_string="DELETE FROM owner WHERE id='$id'";
   $sth = $dbh->Query($query_string)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string\n";
}

################################################################################
#
# Generate the delete page based upon the previously entered key, asking
# for an absoloute commital to the deletion.
#
################################################################################
sub remove_entry
{
   local($entry, @fields)=@_;

   &set_up_page("Delete '$FORM{'Query'}':");
   print <<END
<FORM METHOD="POST"
        ACTION="/cgi-bin/$my_name">
<INPUT TYPE="hidden" name="submit" value="delete">
<INPUT TYPE="hidden" name="delete_value" value=\"$FORM{'Query'}\">
<INPUT TYPE="hidden" name="query_type" value=\"$FORM{'query_type'}\">
END
;
   &display($entry,"display", @fields);
   print <<END
<INPUT TYPE="submit" VALUE="Commit Deletion">
</FORM>
END
;
   &end_page(1);
   exit;
}

################################################################################
#
# Display the entire contents of the database.
#
################################################################################
sub dump_database
{
   local($count, @fields, $bgcolor);

   &set_up_page("Listing of Database Contents");
   @fields=get_db_fields();
   print "<TABLE BORDER=2 CELLPADDING=0 CELLSPACING=0>\n<TR>\n";
   $bgcolor="#EEEEEE";
print "   <TD bgcolor=$bgcolor ALIGN=CENTER><B><FONT SIZE=-1>Delete</FONT></B></TD>\n";

   foreach $count (0..$#fields)
   {
      $location = $count if ($FORM{'query_type'} eq $fields[$count]);
      print "   <TD bgcolor=$bgcolor>\n      <B>$fields[$count]</B>\n   </TD>\n";
   }
   print "</TR>\n";
   if ($FORM{'Range'} eq "Entire")
   {
      %subnets=&get_subnets();
   }
   else
   {
      %subnets=("$FORM{'Range'}");
   }
   foreach $SUBNET (sort numerically keys %subnets)
   {
      if ($FORM{'query_type'} eq "IP")
      {
         &dump_by_ip($SUBNET);
      }
      else
      {
         &dump_else($SUBNET);
      }
   }
   print "</TABLE>\n";
   &end_page(1);
   exit;
}


sub dump_by_ip
{
   local($SUBNET) = @_;
   local($query_string,$query_string2,$sth,$sth2,@row,@row2,$a,$b,$c,$d,%ip_list,$ip_addr,$qs);

   $query_string = "SELECT IP FROM name WHERE IP LIKE '$SUBNET.%'";
   $sth = $dbh->Query($query_string)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   while (@row=$sth->Fetchrow)
   {
      ($a,$b,$c,$d) = split(/\./,$row[0]);
      $temp = &padit($a) . &padit($b) . &padit($c) . &padit($d);
      $ip_list{$temp} = 1;
   }
   foreach $ip_addr (sort numerically keys %ip_list)
   {
   $query_string = "SELECT * FROM name,owner WHERE name.id=owner.id AND IP LIKE '" . &unpadit($ip_addr) . "'";
      $sth = $dbh->Query($query_string)
         or warn "[dump_database] Query had some problem: $Mysql::db_errstr\n$query_string\n";
      @row=$sth->Fetchrow;
      $qs = "SELECT name FROM OS WHERE id='$row[4]'";
      $sth2 = $dbh->Query($qs)
         or warn "[dump_database] Query had some problem: $Mysql::db_errstr\n$qs\n";
      @row2=$sth2->Fetchrow;
      $row[4] = $row2[0];
      $query_string2 = "SELECT * FROM aliases WHERE id='$row[0]'";
      $sth2 = $dbh->Query($query_string2)
         or warn "[dump_database] Query had some problem: $Mysql::db_errstr\n$query_string\n";
      if (@row2=$sth2->Fetchrow)
      {
         $row[5] = $row2[1];   
      }
      else
      {
          $row[5] = "&nbsp;";
      }
      while (@row2=$sth2->Fetchrow)
      {
         $row[5] .= ",<BR>$row2[1]";
      }
      if ($bgcolor eq "#EEEEEE")
      {
         $bgcolor="#FFFFFF";
      }
      else
      {
         $bgcolor="#EEEEEE";
      }
      print "<TR>\n";
      for ($count=0; $count<=$#row; $count++)
      {
         print "   <TD bgcolor=$bgcolor>\n";
         if ($count eq "1")
         {
            print qq#      <A HREF="/cgi-bin/$my_name?query_type=IP&function=Modify&Query=$row[$count]">$row[$count]</A>\n# if ($count == 1);
         }
         else
         {
            if ($count eq "0")
            {
               print qq#      <CENTER><A HREF="/cgi-bin/$my_name?query_type=IP&function=Delete&Query=$row[1]"><IMG SRC="@@IMAGES@@/delete.gif" BORDER=0></A></CENTER>\n#;
            }
            else
            {
               print "$row[$count]&nbsp;\n";
            }
         }
         print "   </TD>\n";
      }
      print "</TR>\n";
   }
}

sub dump_else
{
   local($SUBNET) = @_;
   local($query_string,$query_string2,$sth,$sth2,@row,@row2);

   $query_string = "SELECT * FROM name,owner WHERE name.id=owner.id AND IP LIKE '$SUBNET.%' ORDER BY '$FORM{'query_type'}'";
   $sth = $dbh->Query($query_string)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   while (@row=$sth->Fetchrow)
   {
      $qs = "SELECT name FROM OS WHERE id='$row[4]'";
      $sth2 = $dbh->Query($qs)
         or warn "[dump_database] Query had some problem: $Mysql::db_errstr\n$qs\n";
      @row2=$sth2->Fetchrow;
      $row[4] = $row2[0];
      $query_string2 = "SELECT * FROM aliases WHERE id='$row[0]'";
      $sth2 = $dbh->Query($query_string2)
         or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string\n";
      if (@row2=$sth2->Fetchrow)
      {
         $row[5] = $row2[1];   
      }
      else
      {
          $row[5] = "&nbsp;";
      }
      while (@row2=$sth2->Fetchrow)
      {
         $row[5] .= ",<BR>$row2[1]";
      }
      if ($bgcolor eq "#EEEEEE")
      {
         $bgcolor="#FFFFFF";
      }
      else
      {
         $bgcolor="#EEEEEE";
      }
      print "<TR>\n";
      for ($count=0; $count<=$#row; $count++)
      {
         print "   <TD bgcolor=$bgcolor>\n";
         if ($count eq "0")
         {
            print qq#      <CENTER><A HREF="/cgi-bin/$my_name?query_type=IP&function=Delete&Query=$row[1]"><IMG SRC="@@IMAGES@@/delete.gif" BORDER=0></A></CENTER>\n#;
         }
         else
         {
            if ($count eq "1")
            {
               print qq#      <A HREF="/cgi-bin/$my_name?query_type=IP&function=Modify&Query=$row[$count]">$row[$count]</A>\n#;
            }
            else
            {
               print "$row[$count]&nbsp;\n";
            }
         }
         print "   </TD>\n";
      }
      print "</TR>\n";
   }
}
################################################################################
#
# Tkae the incoming number (an IP), break it apart, pad each with leading
# zeros, and put it together into one numeric string.
#
################################################################################
sub padit
{
   local($num) = @_;

   $num="00" . $num if ($num<10);
   $num="0" . $num if (($num>9) && ($num<100));
   return ($num);
}

################################################################################
#
# Take the incoming number (a padded IP), and break it back down into its
# full IP.
#
################################################################################
sub unpadit
{
   local($num) = @_;

   $a = (substr($num, 0, 3)) + 0;
   $b = (substr($num, 3, 3)) + 0;
   $c = (substr($num, 6, 3)) + 0;
   $d = (substr($num, 9, 3)) + 0;
   return ("$a.$b.$c.$d");
}
   

################################################################################
#
# Get the next available IP for that subnet.
#
################################################################################
sub new_ip
{
   local ($SUBNET) = @_;
   local ($max, $ip);

   $query_string = "SELECT IP FROM name WHERE IP LIKE '$SUBNET.%'";
   $sth = $dbh->Query($query_string)
      or warn "[new_ip] Query had some problem: $Mysql::db_errstr\n$query_string
\n";
   while (@row=$sth->FetchRow)
   {
      ($a,$b,$c,$d) = split(/\./, $row[0]);
      $ip = &padit($a) . &padit($b) . &padit($c) . &padit($d);
      $ips{$ip} = 1 if ($ip != 0);
   }  
   $max=$RANGE[0];
   foreach $addr (sort numerically keys %ips)
   {  
      $ip=substr($addr, 9);
      next if (($ip < $RANGE[0]) || ($ip > $RANGE[1]));
      ($max++) if ($ip == $max);
   }
   if ($max > 254)
   {
      &error_out("The $subnet subnet is full.  Please select another subnet.");
   }
   return ("$SUBNET.$max");
}

################################################################################
#
# Parse the entire database, looking for all of the possible valid subnets,
# and dump them into an array.
#
################################################################################
sub get_subnets
{
   local($sub);

   $query_string="SELECT IP FROM name";
   $sth = $dbh->Query($query_string)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string
\n";
   while (@row=$sth->FetchRow)
   {
      $sub = (split(/\./, $row[0]))[0] . "." . (split(/\./, $row[0]))[1] . "." . (split(/\./, $row[0]))[2];
      $subnets{$sub} = 1 if ($sub != 0);
   }
   return (%subnets);
}

################################################################################
#
# Generate the real DNS files from the master database.  Break out the
# information, in a fully subnet-sorted order, and put them in the hosts
# and reverse database file format.
#
################################################################################
sub update_maps()
{
   local ($first,$second,$third,$fourth,%subnets);

   %subnets=&get_subnets();
   &build_all_dns(%subnets) if ($NAMED_DIR);
   &build_yp_hosts(%subnets) if ($NIS_DIR);
   &build_yp_ethers(%subnets) if ($NIS_DIR);
   &build_dhcp() if ($DHCPD_DIR);
}

################################################################################
#
# Generate the dhcpd table.
#
################################################################################
sub build_dhcp
{
   local($qs);
   $qs = "SELECT dhcpd_header FROM conf WHERE domainname='$DOMAINNAME'"; 
   $sth = $dbh->Query($qs)
      or warn "[build_dhcp] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   $_ = $row[0];
   s/
//g;
   $header = $_;
   $qs = "SELECT IP,machinename,hardware_addr FROM name WHERE hardware_addr!='' AND hardware_addr!='0'";
   $sth = $dbh->Query($qs)
      or warn "[build_dhcp] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   while (@row=$sth->FetchRow)
   {
      $content .= "
host $row[1]
{
   option host-name \"$row[1]\";
   hardware ethernet $row[2];
   fixed-address $row[0];
}
";
   }
   open(DHCP, "> $TMP/dhcpd.conf") || die "Could not write dhcpd.conf\n";
   print DHCP $header;
   print DHCP $content;
   close (DHCP);
}

################################################################################
#
# Grab the header information for YP from the 'conf' database.
#
################################################################################
sub yp_header
{
   local($qs);
   $qs = "SELECT yp_header FROM conf WHERE domainname='$DOMAINNAME'"; 
   $sth = $dbh->Query($qs)
      or warn "[yp_header] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   $_ = $rowp[0];
   s/^M//g;
   print NEWYP $row[0];
}

################################################################################
#
# Build the hosts file, using the header from the database and separating
# each subnet to keep things clean and easy to visually read.
#
################################################################################
sub build_yp_hosts
{
   local(%subnets) = @_;

   open(NEWYP, "> $TMP/hosts") || die &error_out("Could not write temporary hosts file.");
   &yp_header();
   foreach $SUBNET (sort numerically keys %subnets)
   {
      print NEWYP "#------------------------------\n";
      print NEWYP "# Subnet $SUBNET\n";
      print NEWYP "#------------------------------\n";
      $query_string = "SELECT * FROM name WHERE IP LIKE '$SUBNET.%'";
      $sth = $dbh->Query($query_string)
         or warn "[build_yp] Query had some problem: $Mysql::db_errstr\n$query_string\n";
      while (@row=$sth->FetchRow)
      {
         ($a,$b,$c,$d) = split(/\./, $row[1]);
         $ip = &padit($a) . &padit($b) . &padit($c) . &padit($d);
         $ip = $ip . ":" . $row[2] . ":" . $row[0];
         $ips{$ip} = 1 if ($ip != 0);
      }
      foreach $addr (sort numerically keys %ips)
      {
         ($ip, $name, $id) = split(/:/, $addr);
         $aliases = &get_aliases($id);
         $ip = unpadit($ip);
         print NEWYP "$ip	$name $name.$DOMAINNAME";
         foreach $alias (split(/\,/, $aliases))
         {
            print NEWYP " $alias";
         }
         print NEWYP "\n";
         delete $ips{$addr};
      }
   }
   close(NEWYP);
}

################################################################################
#
# Build the ethers file.  Grab everything because it is easier..
#
################################################################################
sub build_yp_ethers
{
   local($qs, @row);

   $qs = "SELECT hardware_addr,machinename FROM name";
   open(NEWYP, "> $TMP/ethers") || die &error_out("Could not write temporary hosts file.");
   $sth = $dbh->Query($qs)
      or warn "[build_yp_ethers] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   while (@row=$sth->FetchRow)
   {
      next if ( !($row[0]) || ($row[0] == 0) );
      print NEWYP "$row[0]	$row[1]\n";
   }
   close(NEWYP);
}

################################################################################
#
# Get the current serial number from the existing DNS files.
#
################################################################################
sub get_dns_serial
{
   local($serial);

   $serial = `cat /var/named/princeton.divatv.com | grep Serial`;
   chop ($serial);
   $serial = (split(/\s+/, (split(/;/, $serial))[0]))[1];
   return ($serial);
}

################################################################################
#
# Reverse the IP and put it in the standard DNS REV format.
#
################################################################################
sub reverse
{
   local($ip) = @_;

   ($a,$b,$c,$d) = split(/\./, $ip);
   return ("$d.$c.$b.$a.IN-ADDR.ARPA.");
}

################################################################################
#
# Get the DNS header information from the 'conf' database, while incrementing
# the serial number accordingly.
#
################################################################################
sub dns_header
{
   local($forward,$serial) = @_;
   local($qs,$rev,$fwd,$_,@row);

   $qs = "SELECT dns_header FROM conf WHERE domainname='$DOMAINNAME'"; 
   $sth = $dbh->Query($qs)
      or warn "[dns_header] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   $_ = $row[0];
   s/\@SERIAL\@/$serial/g;
   s/
//g;
   $fwd = $_;
   $qs = "SELECT dns_fwd_header FROM conf WHERE domainname='$DOMAINNAME'"; 
   $sth = $dbh->Query($qs)
      or warn "[dns_header] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   $_ = $row[0];
   s/
//g;
   $rev = $_;
   print "$fwd";
   if ($forward)
   {
      print "$rev";
   }
   print "\n";
}

################################################################################
#
# Build the DNS forward and reverse files, pulling the configurable information
# from the 'conf' database.
#
################################################################################
sub build_all_dns
{
   local(%subnets) = @_;

   $serial = (&get_dns_serial()) + 1;

   open (FWD, "> $TMP/$DOMAINNAME") || die "Could not write out FWD file.\n";
   select(FWD);
   &dns_header(1,$serial);
   select(STDOUT);
   foreach $SUBNET (sort numerically keys %subnets)
   {
      open(REV, ">$TMP/$SUBNET.rev");
      select(REV);
      &dns_header(0,$serial);
      select(STDOUT);
      $query_string = "SELECT * FROM name WHERE IP LIKE '$SUBNET.%'";
      $sth = $dbh->Query($query_string)
         or warn "[build_yp] Query had some problem: $Mysql::db_errstr\n$query_string\n";
      while (@row=$sth->FetchRow)
      {
         ($a,$b,$c,$d) = split(/\./, $row[1]);
         $ip = &padit($a) . &padit($b) . &padit($c) . &padit($d);
         $ip = $ip . ":" . $row[2] . ":" . $row[0];
         $ips{$ip} = 1 if ($ip != 0);
      }
      print FWD ";------------------------------\n";
      print FWD "; Subnet $SUBNET\n";
      print FWD ";------------------------------\n";
      foreach $addr (sort numerically keys %ips)
      {
         ($ip, $name, $id) = split(/:/, $addr);
         $aliases = &get_aliases($id);
         $ip = unpadit($ip);
         $revip = &reverse($ip);
         select (FWD);
         $~ = DNS_A;
         write;
         select(REV);
         $name.= ".$DOMAINNAME.";
         $~ = DNS_REV;
         write;
         select (STDOUT);
         foreach $alias (split(/\,/, $aliases))
         {
            select (FWD);
            $~ = DNS_C;
            write;
            select(STDOUT);
         }
         delete $ips{$addr};
      }
      close (REV);
   }
   close(FWD);
}

################################################################################
#
# Simply parse the database, breaking out subnets, count them all up, and
# generate a page showing how many IP's are used/available for each subnet.
#
################################################################################
sub subnet_usage
{
   local ($used, $left);

   &set_up_page("IP Usage Statistics");
   %subnets=&get_subnets();
   print <<END
<TABLE BORDER=3>
<TR>
<TD ALIGN=CENTER><B>SUBNET</B></TD>
<TD ALIGN=CENTER><B>IP's USED</B></TD>
<TD ALIGN=CENTER><B>IP's AVAILABLE</B></TD>
</TR>
END
;
   foreach $SUBNET (sort numerically keys %subnets)
   {
      $used = 0;
      print "<TR><TD>$SUBNET</TD>";
      $query_string="SELECT IP FROM name WHERE IP LIKE '$SUBNET.%'";
      $sth = $dbh->Query($query_string)
         or warn "[insert] Query had some problem: $Mysql::db_errstr\n$query_string\n";
      while (@row=$sth->FetchRow)
      {
         $used ++;
      }
      $left = 256-$used;
      print "<TD ALIGN=CENTER>$used</TD><TD ALIGN=CENTER>$left</TD></TR>";
   }
   print "</TABLE>";
   &end_page(1);
   exit;
}

################################################################################
#
# Call the taintperl script which will actually push the generated files to
# the correct machines.
#
################################################################################
sub push_maps
{
   system("@@SUDO@@ @@ADMIN@@/hostupdate > /tmp/output");
}

################################################################################
#
# Define the numerical sorting routine.
#
################################################################################
sub numerically
{
   $a <=> $b;
}

################################################################################
#
# Bring up the administrative interface
#
################################################################################
sub administer
{
   &set_up_page("Administrative Interface");
   $qs = "SELECT dns_header,dns_fwd_header,yp_header,dhcpd_header,named_restart,yp_restart,dhcpd_restart FROM conf WHERE domainname='$DOMAINNAME'";
   $sth = $dbh->Query($qs)
      or warn "[dns_header] Query had some problem: $Mysql::db_errstr\n$query_string\n";
   @row=$sth->FetchRow;
   $dns_header = $row[0];
   $dns_fwd_header = $row[1];
   $nis_header = $row[2];
   $dhcpd_header = $row[3];
   $DNS_RESTART = $row[4];
   $YP_RESTART = $row[5];
   $DHCPD_RESTART = $row[6];

   print <<EOF
<FORM METHOD="POST" ACTION="$my_name">
<INPUT TYPE=HIDDEN NAME="UPDATE_CONF" VALUE="1">
<INPUT TYPE=HIDDEN NAME="DOMAINNAME" VALUE="$DOMAINNAME">
<TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0 WIDTH=100%>
<TR>
   <TD>
      <TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0>
      <TH COLSPAN=2 BGCOLOR="#aeaeae"> Global Settings</TH>
      <TR>
         <TD>Temporary Directory : </TD>
         <TD><INPUT TYPE=TEXT NAME="TMP" VALUE="$TMP" SIZE=20></TD>
      </TR>
      <TR>
         <TD>IP Start Range : </TD>
         <TD><INPUT TYPE=TEXT NAME="RANGE1" VALUE="$RANGE[0]" SIZE=3></TD>
      </TR>
      <TR>
         <TD>IP End Range : </TD>
         <TD><INPUT TYPE=TEXT NAME="RANGE2" VALUE="$RANGE[1]" SIZE=3></TD>
      </TR>
      </TABLE>
   </TD>
   <TD>
      <TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0>
      <TH COLSPAN=2 BGCOLOR="#aeaeae"> Domain Name: $DOMAINNAME</TH>
      <TR>
         <TD>Manage NAMED/BIND : </TD>
         <TD><INPUT TYPE=CHECKBOX NAME="NAMED_ACTIVE"
EOF
;
   print " CHECKED" if ($NAMED_DIR);
   print <<EOF
></TD>
      </TR>
      <TR>
         <TD>Manage NIS/YP : </TD>
         <TD><INPUT TYPE=CHECKBOX NAME="YP_ACTIVE"
EOF
;
   print " CHECKED" if ($NIS_DIR);
   print <<EOF
></TD>
      </TR>
      <TR>
         <TD>Manage DHCPD : </TD>
         <TD><INPUT TYPE=CHECKBOX NAME="DHCPD_ACTIVE"
EOF
;
   print " CHECKED" if ($DHCPD_DIR);
   print <<EOF
></TD>
      </TR>
      </TABLE>
   </TD>
</TR>
<TR>
   <TD COLSPAN=2>
      <TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0>
      <TH COLSPAN=2 BGCOLOR="#aeaeae"> NAMED/BIND Configuration</TH>
      <TR>
         <TD>File Locations : </TD>
         <TD><INPUT TYPE=TEXT NAME="NAMED_DIR" VALUE="$NAMED_DIR" SIZE=20></TD>
      </TR>
      <TR>
         <TD>Header for Forward File : </TD>
         <TD><TEXTAREA NAME="dns_header" ROWS=5 COLS=80>$dns_header</TEXTAREA></TD>
      </TR>
      <TR>
         <TD>Header for Reverse File : </TD>
         <TD><TEXTAREA NAME="dns_fwd_header" ROWS=5 COLS=80>$dns_fwd_header</TEXTAREA></TD>
      </TR>
      <TR>
         <TD>Restart/Reload Command : </TD>
         <TD><INPUT TYPE=TEXT NAME="DNS_RESTART" VALUE='$DNS_RESTART' SIZE=30></TD>
      </TR>
      </TABLE>
   </TD>
</TR>
<TR>
   <TD COLSPAN=2>
      <TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0>
      <TH COLSPAN=2 BGCOLOR="#aeaeae"> NIS/YP Configuration</TH>
      <TR>
         <TD>File Locations : </TD>
         <TD><INPUT TYPE=TEXT NAME="NIS_DIR" VALUE="$NIS_DIR" SIZE=20></TD>
      </TR>
      <TR>
         <TD>Header for Files : </TD>
         <TD><TEXTAREA NAME="nis_header" ROWS=5 COLS=80>$nis_header</TEXTAREA></TD>
      </TR>
      <TR>
         <TD>Restart/Reload Command : </TD>
         <TD><INPUT TYPE=TEXT NAME="YP_RESTART" VALUE='$YP_RESTART' SIZE=30></TD>
      </TR>
      </TABLE>
   </TD>
</TR>
<TR>
   <TD COLSPAN=2>
      <TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0>
      <TH COLSPAN=2 BGCOLOR="#aeaeae"> DHCPD Configuration</TH>
      <TR>
         <TD>File Locations : </TD>
         <TD><INPUT TYPE=TEXT NAME="DHCPD_DIR" VALUE="$DHCPD_DIR" SIZE=20></TD>
      </TR>
      <TR>
         <TD>Header for Files : </TD>
         <TD><TEXTAREA NAME="dhcpd_header" ROWS=5 COLS=80>$dhcpd_header</TEXTAREA></TD>
      </TR>
      <TR>
         <TD>Restart/Reload Command : </TD>
         <TD><INPUT TYPE=TEXT NAME="DHCPD_RESTART" VALUE='$DHCPD_RESTART' SIZE=30></TD>
      </TR>
      </TABLE>
      <HR SIZE=1 NOSHADE>
   </TD>
</TR>
<TR>
   <TD ALIGN=CENTER>
      <TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0>
      <TR>
         <TD>
            <INPUT TYPE=SUBMIT NAME="Update" VALUE="Save Changes">
         </TD>
         <TD>
            <INPUT TYPE=SUBMIT NAME="Update" VALUE="Cancel">
         </TD>
      </TR>
      </TABLE>
   </TD>
</TR>
</TABLE>
</FORM>
EOF
;
}

################################################################################
#
# Update the configuration database
#
################################################################################
sub update_config
{
   $FORM{'NAMED_DIR'} = "" if (! $FORM{'NAMED_ACTIVE'});
   $FORM{'NIS_DIR'} = "" if (! $FORM{'YP_ACTIVE'});
   $FORM{'DHCPD_DIR'} = "" if (! $FORM{'DHCPD_ACTIVE'});
   $qs="UPDATE conf set named='$FORM{'NAMED_DIR'}',yp='$FORM{'NIS_DIR'}',dhcpd='$FORM{'DHCPD_DIR'}',tmp='$FORM{'TMP'}',start_range='$FORM{'RANGE1'}',end_range='$FORM{'RANGE2'}',dns_header='$FORM{'dns_header'}',dns_fwd_header='$FORM{'dns_fwd_header'}',yp_header='$FORM{'nis_header'}',dhcpd_header='$FORM{'dhcpd_header'}',named_restart='$FORM{'DNS_RESTART'}',yp_restart='$FORM{'YP_RESTART'}',dhcpd_restart='$FORM{'DHCPD_RESTART'}' WHERE domainname='$FORM{'DOMAINNAME'}'";
   $sth = $dbh->Query($qs)
      or warn "[update_config] Query had some problem: $Mysql::db_errstr\n$qs\n";
}

################################################################################
#
# Show a standard error screen if we happen to hit any odd errors when opening
# or writing to any files.
#
################################################################################
sub error_out
{
   local($error) = @_;

   &set_up_page("Unexpected Error!");
   print <<END
There has been an unexpected server-side error.  Please contact the
Administrator to determine the cause of the error.<BR>
<BR>
<B>ERROR: $error</B>
END
;
   &end_page(1);
   exit;
}

################################################################################
#
# Do a basic check to make sure the entry is not null.
# (This is to be replaced with a more thorough check which really looks
# (at the IP, hostname, and aliases to make sure there is no error there.)
#
################################################################################
sub check_entry
{
   local(@fields) = @_;

   foreach $count (0..$#fields)
   {
      if ($FORM{$field[$count]} eq "")
      {
         $error = 1;
      }
   }
   return ($error);
}

################################################################################
#
# Display a status notice after each change to the database.
#
################################################################################
sub notify
{
   local($notice) = @_;

   if ($notice eq 1)
   {
      print <<END
<B><BLINK>ERROR</BLINK></B>: No such entry, "
END
;
      print "$FORM{'Query'}" if ($FORM{'Query'});
      print "$FORM{'Delete'}" if ($FORM{'Delete'});
      print "$FORM{'Modify'}" if ($FORM{'Modify'});
   print <<END
", found in the database under "$FORM{'query_type'}".  Please re-submit.<BR>
END
;
   }
   if ($error eq 2)
   {
      print "<B><BLINK>NOTICE</BLINK></B>: ";
      print "$FORM{$field[1]} has been <B>deleted</B> from the database.<BR>" if ($FORM{'submit'} eq "delete");
      print "$FORM{$field[1]} has been updated.<BR>" if ($FORM{'submit'} eq "modify");
      print "$FORM{$field[1]} has been added with the IP address of <B>$FORM{'IP'}</B><BR>" if ($FORM{'submit'} eq "add");
   }
   if ($error eq 3)
   {
      print "<B><BLINK>NOTICE</BLINK></B>: ";
      print "The maps have been <B><I>PUSHED</I></B>.<BR>";
   }
   if ($error eq 4)
   {
      print "<B><BLINK>NOTICE</BLINK></B>: ";
      print "The configuration settings have been <B><I>UPDATED</I></B>.<BR>";
   }
   if ($error eq 5)
   {
      print "<B><BLINK>NOTICE</BLINK></B>: ";
      print "The configuration changes have been <B><I>Cancelled</I></B>. No Changes have been made.<BR>";
   }
   if ($error eq 20)
   {
      print "<B><FONT COLOR=\"RED\"><BLINK>ERROR</BLINK>:</FONT></B> ";
      print "That machine name is already in use.<BR>";
   }
   if ($error eq 21)
   {
      print "<B><FONT COLOR=\"RED\"><BLINK>ERROR</BLINK>:</FONT></B> ";
      print "That IP address is already in use.<BR>";
   }
   if ($error eq 22)
   {
      print "<B><FONT COLOR=\"RED\"><BLINK>ERROR</BLINK>:</FONT></B> ";
      print "That hardware address is already in use.<BR>";
   }
   if ($error eq 23)
   {
      print "<B><FONT COLOR=\"RED\"><BLINK>ERROR</BLINK>:</FONT></B> ";
      print "No machine name or IP address was specified.<BR>";
   }
}

################################################################################
#
# Give some help information.
#
################################################################################
sub help
{
   local($describe) = @_;
   local($qs);

   $qs = "SELECT info FROM descriptions WHERE item='$describe'";
   $sth = $dbh->Query($qs)
      or warn "[defaults] Query had some problem: $Mysql::db_errstr\n$qs\n";
   @row=$sth->FetchRow;
   print "$row[0]";
}

################################################################################
#
# Check to see if the item exists in the database already.
#
################################################################################
sub exists
{
   local($item,$field) = @_;
   local($sth,$qs,$exists,@row);

   $qs = "SELECT * FROM name WHERE $field='$item'";
   $sth = $dbh->Query($qs)
      or warn "[defaults] Query had some problem: $Mysql::db_errstr\n$qs\n";
   @row=$sth->FetchRow;
   $exists = 1 if ($row[0]);
   return ($exists);
}
################################################################################
#
# Define the error/notification types.
#
################################################################################
sub errors
{
   local(@fields) = @_;
   local($count,$error);

   foreach $count (0..$#fields)
   {
      if (!$fields[$count])
      {
         $error=1;
      }
   }
#   if ($FORM{'submit'} eq "modify")
#   {
#      &check_ip();
#      &check_name();
#      &check_hostid();
#      &check_aliases();
#   }
#   if ($FORM{'submit'} eq "add")
#   {
#      &check_ip() if ($explicit);
#      &check_name();
#      &check_hostid();
#      &check_aliases();
#   }
   return ($error);
}

################################################################################
#
# Display the change log on-line rather than forcing the user to search for
# and read a particular file.
#
################################################################################
sub show_changelog
{
   $changes = <<EOF
1.8
- Created the Makefile
- Changed the source directory structure for easier installation/upgrade
- Replaced all key variables in the source files with flags which get
  replaced upon installation/upgrade via the Makefile.
- Created the README and INSTALL text files.
- Updated the hostupdate script to pull all of its configuration information
  out of the database.
- Added in the restart command sequences for named, yp, and dhcpd to make it
  fully configurable.
- Created the administrative interface layout.
- Properly populated the layout.
- The Administrative interface now updates the configuration file.
- Added the proper notification messages for updating and cancelling the updates
  to the configuration database.
- Put in hooks for global on/off switches for DNS, DHCP, and YP on the admin
  page and used them to set/clear the directory setting for the particular
  service.  The program checks to see if that directory setting is set to
  anything to determine if we should process the files for that service.
- Fixed a minor typo in the error listing.  I accidentally listed error#21
  twice and left off error#22.
- Created error#23 for situations when a hostname or IP is not specified.  We
  cannot have blank entries in the resulting files.

1.7
- Added in the description graphic.
- Added in the javascript to handle the pop-ups.
- Created the "descriptions" table in the database.
- Added the "help" routine to handle when the description graphic is clicked on.
- Populated the "descriptions" table.
- Removed the two "NOTE" lines because of the new help format.
- Added in some rudimentary error checking when adding a new host.  It checks
  the hostname, IP, and hardware address to see if any of them are in use.  If
  they are, it generates the appropriate error message.
- Added the configuration aspect for DHCP.
- Added in the "comment" and "OS" fields.  "OS" is a little special because of
  the pull-down selection capability.
- Fixed a bug in the display by IP function.  It was something I fixed some
  time ago in the display by other function, but, oddly, I never made the same
  change.  Hence, things broke in funny ways when I added the two new fields.
- Started creating the Administration interface.  It barely shows anything and
  you cannot do anything with it yet.

1.6
- Added comments to the uncommented sections of the code.
- Updated the initial description at the top of the script.
- Broke apart the dump_database routine.  Hopefully, this will make
  it easier to build the sort functions for the other columns.
- Sorting now sort-of works on the other columns (except aliases).  I need to
  work on the SQL query a bit more because of the way the join is working.
- Querying by ownername and location now works.  Keep in mind that this will
  only result in the first record that matches.
- New Graphic.  The program is a bit more than just a DNS database now..
- Created a main table for all of the options.
- Added in the creation of the ethers NIS map.  This was specifically done
  to support the use of Solaris's jumpstart.
- Added logout capability.
- Code now always adds to all of the database fields, even if there is no
  content.  This makes searching a whole heck of a lot easier.
- Redid the login screen.  Created a graphical "badge" that needs to be
  filled in.
- Created a hook for the administrative interface, but I have not built it yet.
- Fixed a bug in the display of the Entire database when sorting by IP where
  it would keep adding to the entries to be displayed.  Not that it existed
  when displaying other items, I reflected the bug change there, as well, in
  the event that we change something down the road.
- Fixed the display of the "log out" option at the bottom of the screen and
  increased the delay of the "Processing Log Out..." option to allow for
  the cookie to be completely destroyed and for the browser to notice its
  removal.

1.5
- Databased all of the configuration information.  This will build a foundation
  for a web-based configuration page.
- Databased all of the file header information.  This, too, will allow for
  a web-based customization page.
- Hacked my script to handle the restarting of the dhcp daemon, which runs on
  another server.
- Added this on-line changelog routine. :)

1.4 (never in production)
- Cleaned up the DHCPd configuration options.  It should set the NIS domain
  name amongst other information.
- Fixed the DNS TTL flag.
- Other changes that I forgot about over the Xmas break.

1.3
- Cleaned up the interface.  Folded a lot of the original interface into one
  pull-down form.
- Fixed the sort routine for the IP's.  Basically, I had to break apart the
  IP address on the fly and sort it that way.  A bit expensive, but it works.
  Sorting still only works for the first three fields. :(
- Wrote the DHCPd routines.  Restarting the dhcp daemon by hand is still
  necessary.
- Created the cookie interface to the script.

1.2
- Added in some error checking.  Certain people were using invalid DNS
  characters for hostnames, despite my pleas.  I dropped them all into an
  array, and simply remove them from the hostname.  I should fully fill out
  the array soon.
- Added some (read that as "very minimal") on-line documentation.
- Created a graphic for the top of the page.
- Since people were confused by the lack of a "submit" button, I created
  a "go" graphic and put that into the form.
- Cleaned up the file output for the DNS and NIS services.
- Installed sudo.
- Moved the ownership of the scripts and such over to the "janitor" account,
  and wrote a script, to which the janitor account has access to execute, which
  moves all of the files into place.
- Re-did the login screen.

1.1
- Query/sort via IP, machinename, or hardware adress.
- Added a little error checking, but not much.
- Turned on DNS and NIS map creation and rolled it into production.
- Minor bug fixes per request.
- Migrated the login information over to a database.

1.0
- Migration from the old code that I wrote for Sarnoff.
- Got it working under the Diva setup.
- Modified to make it a little more portable.
- Migrated from a flat-file database to a MySQL database.
EOF
;
   print "<PRE>\n";
   print "$changes\n";
   print "</PRE>\n";
}
